@@ -1,4 +1,4 @@
-/* xscreensaver, Copyright (c) 1993-2004 Jamie Zawinski <jwz@jwz.org>
+/* xscreensaver, Copyright (c) 1993-2005 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -97,7 +97,9 @@ struct saver_info {
   int keyboard_grab_screen;	/* The screen number the keyboard grab is on */
   Bool fading_possible_p;	/* Whether fading to/from black is possible. */
   Bool throttled_p;             /* Whether we should temporarily just blank
-                                   the screen, not run hacks. */
+                                   the screen, not run hacks. (Deprecated:
+                                   users should use "xset dpms force off"
+                                   instead.) */
   time_t blank_time;		/* The time at which the screen was blanked
                                    (if currently blanked) or unblanked (if
                                    not blanked.) */
@@ -268,6 +270,7 @@ struct saver_screen_info {
   int poll_mouse_last_root_y;
   Window poll_mouse_last_child;
   unsigned int poll_mouse_last_mask;
+  time_t poll_mouse_last_time;
 
 
   /* =======================================================================
@@ -414,6 +417,7 @@ extern void hack_environment (saver_info *si);
 extern void hack_subproc_environment (saver_screen_info *ssi);
 extern void init_sigchld (void);
 extern void spawn_screenhack (saver_info *si, Bool first_time_p);
+extern pid_t fork_and_exec (saver_screen_info *ssi, const char *command);
 extern void kill_screenhack (saver_info *si);
 extern void suspend_screenhack (saver_info *si, Bool suspend_p);
 extern Bool screenhack_running_p (saver_info *si);
