@@ -19,9 +19,8 @@
     * other special, indirect and consequential damages.
  */
 
-#if !defined( lint ) && !defined( SABER )
+#if 0
 static const char sccsid[] = "@(#)bouboule.c	4.00 97/01/01 xlockmore";
-
 #endif
 
 /*-
