@@ -1,9 +1,8 @@
 /* -*- Mode: C; tab-width: 4 -*- */
 /* euler2d --- 2 Dimensional Incompressible Inviscid Fluid Flow */
 
-#if !defined( lint ) && !defined( SABER )
+#if 0
 static const char sccsid[] = "@(#)euler2d.c	5.00 2000/11/01 xlockmore";
-
 #endif
 
 /*
@@ -69,22 +68,20 @@ static float power = 1;
 
 static XrmOptionDescRec opts[] =
 {
-  {(char* ) "-eulertail", (char *) ".euler2d.eulertail",
-   XrmoptionSepArg, (caddr_t) NULL},
-  {(char* ) "-eulerpower", (char *) ".euler2d.eulerpower",
-   XrmoptionSepArg, (caddr_t) NULL},
+  {"-eulertail", ".euler2d.eulertail",   XrmoptionSepArg, NULL},
+  {"-eulerpower", ".euler2d.eulerpower", XrmoptionSepArg, NULL},
 };
 static argtype vars[] =
 {
-  {(caddr_t *) &tail_len, (char *) "eulertail",
-   (char *) "EulerTail", (char *) DEF_EULERTAIL, t_Int},
-  {(caddr_t *) &power, (char *) "eulerpower",
-   (char *) "EulerPower", (char *) "1", t_Float},
+  {&tail_len, "eulertail",
+   "EulerTail", (char *) DEF_EULERTAIL, t_Int},
+  {&power, "eulerpower",
+   "EulerPower", "1", t_Float},
 };
 static OptionStruct desc[] =
 {
-  {(char *) "-eulertail len", (char *) "Length of Euler2d tails"},
-  {(char *) "-eulerpower power", (char *) "power of interaction law for points for Euler2d"},
+  {"-eulertail len", "Length of Euler2d tails"},
+  {"-eulerpower power", "power of interaction law for points for Euler2d"},
 };
 
 ModeSpecOpt euler2d_opts =
