@@ -1,9 +1,8 @@
 /* -*- Mode: C; tab-width: 4 -*- */
 /* cage --- the Impossible Cage, an Escher like scene. */
 
-#if !defined( lint ) && !defined( SABER )
+#if 0
 static const char sccsid[] = "@(#)cage.c	5.01 2001/03/01 xlockmore";
-
 #endif
 
 /*-
@@ -76,14 +75,6 @@ static const char sccsid[] = "@(#)cage.c	5.01 2001/03/01 xlockmore";
  * In real OpenGL, PseudoColor DO NOT support texture map (as far as I know).
  */
 
-#ifdef VMS
-/*-
- * due to a Bug/feature in VMS X11/Intrinsic.h has to be placed before xlock.
- * otherwise caddr_t is not defined correctly
- */
-#include <X11/Intrinsic.h>
-#endif
-
 #ifdef STANDALONE
 # define MODE_cage
 # define PROGCLASS			"Cage"
@@ -357,7 +348,7 @@ pinit(ModeInfo *mi)
                                  GL_RGB, GL_UNSIGNED_BYTE, WoodTextureData);
 	if (status)
 	  {
-		const char *s = gluErrorString (status);
+		const char *s = (char *) gluErrorString (status);
 		fprintf (stderr, "%s: error mipmapping texture: %s\n",
 				 progname, (s ? s : "(unknown)"));
 		exit (1);
