@@ -2,7 +2,7 @@
 /* gflux - creates a fluctuating 3D grid 
  * requires OpenGL or MesaGL
  * 
- * Copyright (c) Josiah Pease, 2000
+ * Copyright (c) Josiah Pease, 2000, 2003
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
  * the above copyright notice appear in all copies and that both that
@@ -39,35 +39,18 @@
  */
 
 
-/*-
- * due to a Bug/feature in VMS X11/Intrinsic.h has to be placed before xlock.
- * otherwise caddr_t is not defined correctly
- */
-
-#include <X11/Intrinsic.h>
-
-
 #ifdef STANDALONE
 # define PROGCLASS						"gflux"
 # define HACK_INIT						init_gflux
 # define HACK_DRAW						draw_gflux
 # define HACK_RESHAPE					reshape_gflux
+# define HACK_HANDLE_EVENT				gflux_handle_event
+# define EVENT_MASK						PointerMotionMask
 # define gflux_opts						xlockmore_opts
 #define DEFAULTS                        "*delay:		20000   \n" \
 										"*showFPS:      False   \n" \
                                         "*mode:         light" "\n" \
-                                        "*squares:      19      \n" \
-										"*resolution:   0       \n" \
-                                        "*flat:         0       \n" \
-                                        "*speed:        0.05    \n" \
-                                        "*rotationx:    0.01    \n" \
-                                        "*rotationy:    0.0     \n" \
-                                        "*rotationz:    0.1     \n" \
-                                        "*waves:        3       \n" \
-                                        "*waveChange:   50      \n" \
-                                        "*waveHeight:  0.8      \n" \
-                                        "*waveFreq:    3.0      \n" \
-                                        "*zoom:        1.0      \n" 
+                                        "*useSHM:      True     \n" 
 
 
 # include "xlockmore.h"				/* from the xscreensaver distribution */
@@ -85,10 +68,6 @@
 # endif /* VMS */
 #endif
 
-#ifdef HAVE_PPM
-#include <ppm.h>
-#endif
-
 #undef countof
 #define countof(x) (sizeof((x))/sizeof((*x)))
 
@@ -102,9 +81,10 @@
 #include <math.h>
 
 #include "grab-ximage.h"
+#include "gltrackball.h"
 
 
-static enum {wire=0,solid,light,checker,textured,grab} _draw; /* draw style */
+static enum {wire=0,solid,light,checker,grab} _draw; /* draw style */
 static int _squares = 19;                                 /* grid size */
 static int _resolution = 4;                    /* wireframe resolution */
 static int _flat = 0;
@@ -120,41 +100,44 @@ static int _waveChange = 50;
 static float _waveHeight = 1.0;
 static float _waveFreq = 3.0;
 
+static trackball_state *trackball;
+static Bool button_down_p = False;
+
 #define WIDTH 320
 #define HEIGHT 240
 
 static XrmOptionDescRec opts[] = {
-    {"-squares", ".gflux.squares", XrmoptionSepArg, (caddr_t) NULL},
-    {"-resolution", ".gflux.resolution", XrmoptionSepArg, (caddr_t) NULL},
-/*    {"-draw", ".gflux.draw", XrmoptionSepArg, (caddr_t) NULL},*/
-    {"-mode", ".gflux.mode", XrmoptionSepArg, (caddr_t) NULL},
-    {"-flat", ".gflux.flat", XrmoptionSepArg, (caddr_t) NULL},
-    {"-speed", ".gflux.speed", XrmoptionSepArg, (caddr_t) NULL},
-    {"-rotationx", ".gflux.rotationx", XrmoptionSepArg, (caddr_t) NULL},
-    {"-rotationy", ".gflux.rotationy", XrmoptionSepArg, (caddr_t) NULL},
-    {"-rotationz", ".gflux.rotationz", XrmoptionSepArg, (caddr_t) NULL},
-    {"-waves", ".gflux.waves", XrmoptionSepArg, (caddr_t) NULL},
-    {"-waveChange", ".gflux.waveChange", XrmoptionSepArg, (caddr_t) NULL},
-    {"-waveHeight", ".gflux.waveHeight", XrmoptionSepArg, (caddr_t) NULL},
-    {"-waveFreq", ".gflux.waveFreq", XrmoptionSepArg, (caddr_t) NULL},
-    {"-zoom", ".gflux.zoom", XrmoptionSepArg, (caddr_t) NULL},
+    {"-squares", ".gflux.squares", XrmoptionSepArg, 0},
+    {"-resolution", ".gflux.resolution", XrmoptionSepArg, 0},
+/*    {"-draw", ".gflux.draw", XrmoptionSepArg, 0},*/
+    {"-mode", ".gflux.mode", XrmoptionSepArg, 0},
+    {"-flat", ".gflux.flat", XrmoptionSepArg, 0},
+    {"-speed", ".gflux.speed", XrmoptionSepArg, 0},
+    {"-rotationx", ".gflux.rotationx", XrmoptionSepArg, 0},
+    {"-rotationy", ".gflux.rotationy", XrmoptionSepArg, 0},
+    {"-rotationz", ".gflux.rotationz", XrmoptionSepArg, 0},
+    {"-waves", ".gflux.waves", XrmoptionSepArg, 0},
+    {"-waveChange", ".gflux.waveChange", XrmoptionSepArg, 0},
+    {"-waveHeight", ".gflux.waveHeight", XrmoptionSepArg, 0},
+    {"-waveFreq", ".gflux.waveFreq", XrmoptionSepArg, 0},
+    {"-zoom", ".gflux.zoom", XrmoptionSepArg, 0},
 };
 
 
 static argtype vars[] = {
-    {(caddr_t *) & _squares, "squares", "Squares", "19", t_Int},
-    {(caddr_t *) & _resolution, "resolution", "Resolution", "4", t_Int},
-/*    {(caddr_t *) & _draw, "draw", "Draw", "2", t_Int},*/
-    {(caddr_t *) & _flat, "flat", "Flat", "0", t_Int},
-    {(caddr_t *) & _speed, "speed", "Speed", "0.05", t_Float},
-    {(caddr_t *) & _rotationx, "rotationx", "Rotationx", "0.01", t_Float},
-    {(caddr_t *) & _rotationy, "rotationy", "Rotationy", "0.0", t_Float},
-    {(caddr_t *) & _rotationz, "rotationz", "Rotationz", "0.1", t_Float},
-    {(caddr_t *) & _waves, "waves", "Waves", "3", t_Int},
-    {(caddr_t *) & _waveChange, "waveChange", "WaveChange", "50", t_Int},
-    {(caddr_t *) & _waveHeight, "waveHeight", "WaveHeight", "1.0", t_Float},
-    {(caddr_t *) & _waveFreq, "waveFreq", "WaveFreq", "3.0", t_Float},
-    {(caddr_t *) & _zoom, "zoom", "Zoom", "1.0", t_Float},
+    {&_squares, "squares", "Squares", "19", t_Int},
+    {&_resolution, "resolution", "Resolution", "4", t_Int},
+/*    {&_draw, "draw", "Draw", "2", t_Int},*/
+    {&_flat, "flat", "Flat", "0", t_Int},
+    {&_speed, "speed", "Speed", "0.05", t_Float},
+    {&_rotationx, "rotationx", "Rotationx", "0.01", t_Float},
+    {&_rotationy, "rotationy", "Rotationy", "0.0", t_Float},
+    {&_rotationz, "rotationz", "Rotationz", "0.1", t_Float},
+    {&_waves, "waves", "Waves", "3", t_Int},
+    {&_waveChange, "waveChange", "WaveChange", "50", t_Int},
+    {&_waveHeight, "waveHeight", "WaveHeight", "1.0", t_Float},
+    {&_waveFreq, "waveFreq", "WaveFreq", "3.0", t_Float},
+    {&_zoom, "zoom", "Zoom", "1.0", t_Float},
 };
 
 
@@ -198,26 +181,17 @@ typedef struct {
     double dispy[MAXWAVES];
     double dispx[MAXWAVES];
     GLfloat colour[3];
-    int imageWidth;
-    int imageHeight;
-#ifdef HAVE_PPM
-	pixval imageMax;
-    pixel **image;
-#else
-	int imageMax;
-	GLubyte *image;
-#endif
-    GLint texName;
+    GLuint texName;
     GLfloat tex_xscale;
     GLfloat tex_yscale;
+    XRectangle img_geom;
+    int img_width, img_height;
     void (*drawFunc)(void);
 } gfluxstruct;
 static gfluxstruct *gflux = NULL;
 
 /* prototypes */
 void initLighting(void);
-void initTexture(void);
-void loadTexture(void);
 void grabTexture(void);
 void createTexture(void);
 void displaySolid(void);            /* drawFunc implementations */
@@ -240,6 +214,51 @@ double getGrid(double,double,double);
 /* BEGINNING OF FUNCTIONS */
 
 
+Bool
+gflux_handle_event (ModeInfo *mi, XEvent *event)
+{
+  if (event->xany.type == ButtonPress &&
+      event->xbutton.button == Button1)
+    {
+      button_down_p = True;
+      gltrackball_start (trackball,
+                         event->xbutton.x, event->xbutton.y,
+                         MI_WIDTH (mi), MI_HEIGHT (mi));
+      return True;
+    }
+  else if (event->xany.type == ButtonRelease &&
+           event->xbutton.button == Button1)
+    {
+      button_down_p = False;
+      return True;
+    }
+  else if (event->xany.type == ButtonPress &&
+           (event->xbutton.button == Button4 ||
+            event->xbutton.button == Button5))
+    {
+      gltrackball_mousewheel (trackball, event->xbutton.button, 10,
+                              !!event->xbutton.state);
+      return True;
+    }
+  else if (event->xany.type == MotionNotify &&
+           button_down_p)
+    {
+      gltrackball_track (trackball,
+                         event->xmotion.x, event->xmotion.y,
+                         MI_WIDTH (mi), MI_HEIGHT (mi));
+      return True;
+    }
+
+  return False;
+}
+
+
+static void
+userRot(void)
+{
+  gltrackball_rotate (trackball);
+}
+
 /* draw the gflux once */
 void draw_gflux(ModeInfo * mi)
 {
@@ -300,21 +319,12 @@ void initializeGL(ModeInfo *mi, GLsizei width, GLsizei height)
       gflux->drawFunc = (displayTexture);
       glEnable(GL_DEPTH_TEST);
       createTexture();
-      initTexture();
-      initLighting();
-    break;
-	case textured :
-      gflux->drawFunc = (displayTexture);
-      glEnable(GL_DEPTH_TEST);
-      loadTexture();
-      initTexture();
       initLighting();
     break;
 	case grab :
       gflux->drawFunc = (displayTexture);
       glEnable(GL_DEPTH_TEST);
       grabTexture();
-      initTexture();
       initLighting();
     break;
     case wire :
@@ -343,6 +353,8 @@ void init_gflux(ModeInfo * mi)
     }
     gp = &gflux[screen];
 
+    trackball = gltrackball_init ();
+
     {
       char *s = get_string_resource ("mode", "Mode");
       if (!s || !*s)                       _draw = wire;
@@ -350,7 +362,6 @@ void init_gflux(ModeInfo * mi)
       else if (!strcasecmp (s, "solid"))   _draw = solid;
       else if (!strcasecmp (s, "light"))   _draw = light;
       else if (!strcasecmp (s, "checker")) _draw = checker;
-      else if (!strcasecmp (s, "stdin"))   _draw = textured;
       else if (!strcasecmp (s, "grab"))    _draw = grab;
       else
         {
@@ -375,7 +386,6 @@ void init_gflux(ModeInfo * mi)
 /* cleanup code */
 void release_gflux(ModeInfo * mi)
 {
-    if(gflux->image!=NULL) free(gflux->image);
     if(gflux->glx_context!=NULL) free(gflux->glx_context);
     if (gflux != NULL) {
         (void) free((void *) gflux);
@@ -384,272 +394,55 @@ void release_gflux(ModeInfo * mi)
     FreeAllGL(mi);
 }
 
-#ifdef HAVE_PPM
 
-/* load pnm from stdin using pnm libs */
-void loadTexture(void)
-{
-    FILE *file = stdin;
-	gflux->image = ppm_readppm( file, 
-			&(gflux->imageHeight), &(gflux->imageWidth), &(gflux->imageMax) );
-}
-
-/* creates an image for texture mapping */
-void createTexture(void)
-{
-    int i,j,c;
-    pixel **result;
-
-	gflux->imageHeight = gflux->imageWidth = 8;
-
-	result = ppm_allocarray(gflux->imageHeight,gflux->imageWidth);
-    for(i=0;i<gflux->imageHeight;i++) {
-        for(j=0;j<gflux->imageWidth;j++) {
-            c = (((i)%2 ^ (j)%2) ? 100 : 200 );
-			PPM_ASSIGN( result[i][j] , c, c, c );
-        }
-    }
-	gflux->image = result;
-}
-
-/* specifies image as texture */    
-void initTexture(void)
-{
-	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
-	glGenTextures(1, &gflux->texName);
-	glBindTexture(GL_TEXTURE_2D, gflux->texName);
-	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
-	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
-	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
-	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
-
-    clear_gl_error();
-	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, gflux->imageWidth,
-			gflux->imageHeight, 0, GL_RGB, GL_UNSIGNED_BYTE, *(gflux->image));
-    check_gl_error("texture");
-}
-
-#else /* HAVE_PPM FALSE */
-
-#define presult(A,B,C) (*(result+(A)*(gflux->imageWidth)*4+(B)*4+(C)))
-void loadTexture(void)
-{
-    int i, j, levels, width, height;
-    int red,green,blue;
-    char s[4];
-    int ppmType=0;
-    FILE *file = stdin;
-    GLubyte *result;
-
-    fgets(s,4,file);
-
-    if(!strncmp(s,"P6",2)) ppmType=6;
-    if(!strncmp(s,"P5",2)) ppmType=5;
-    if(!strncmp(s,"P3",2)) ppmType=3;
-    if(!strncmp(s,"P2",2)) ppmType=2;
-    if(!ppmType)exit(1);
-
-    while((i=getc(file))=='#')
-    {
-        while(getc(file)!='\n');
-    }
-    ungetc(i,file);
-
-    fscanf(file,"%d %d %d",&width,&height,&levels);
-
-    result = malloc(sizeof(GLubyte)*4*width*height);
-    gflux->imageWidth = width;
-    gflux->imageHeight = height;
-
-    switch(ppmType) {
-        case 2 :    /* ASCII grey */
-            for(i=0;i<height;i++) {
-                for(j=0;j<width;j++) {
-                    fscanf(file,"%d",&red);
-                    presult(j,i,0) = red;
-                    presult(j,i,1) = red;
-                    presult(j,i,2) = red;
-                }
-            }
-            break;
-        case 3 :    /* ASCII rgb */
-            for(i=0;i<height;i++) {
-                for(j=0;j<width;j++) {
-                   fscanf(file,"%d %d %d",&red,&green,&blue);
-                    presult(j,i,0) = red;
-                    presult(j,i,1) = green;
-                    presult(j,i,2) = blue;
-                }
-            }
-            break;
-        case 5 :    /* Binary grey */
-            getc(file); /* seems nessessary */
-            for(i=0;i<height;i++) {
-                for(j=0;j<width;j++) {
-                    red = getc(file);
-                    presult(j,i,0) = red;
-                    presult(j,i,1) = red;
-                    presult(j,i,2) = red;
-                }
-            }
-        break;
-        case 6 :    /* Binary rgb */
-            getc(file); /* seems nessessary */
-            for(i=0;i<height;i++) {
-                for(j=0;j<width;j++) {
-                    red = getc(file);
-                    green = getc(file);
-                    blue = getc(file);
-                    presult(j,i,0) = red;
-                    presult(j,i,1) = green;
-                    presult(j,i,2) = blue;
-                }
-            }
-        break;
-    }
-    gflux->image = result;
-}
 
 void createTexture(void)
 {
-    int i,j,c;
-    GLubyte *result;
+  int size = 4;
+  unsigned int data[] = { 0xFFFFFFFF, 0xAAAAAAAA, 0xFFFFFFFF, 0xAAAAAAAA,
+                          0xAAAAAAAA, 0xFFFFFFFF, 0xAAAAAAAA, 0xFFFFFFFF,
+                          0xFFFFFFFF, 0xAAAAAAAA, 0xFFFFFFFF, 0xAAAAAAAA,
+                          0xAAAAAAAA, 0xFFFFFFFF, 0xAAAAAAAA, 0xFFFFFFFF };
 
-    gflux->imageHeight = gflux->imageWidth = 8;
+  gflux->tex_xscale = size;
+  gflux->tex_yscale = size;
 
-    result = malloc(sizeof(GLubyte)*4*gflux->imageHeight*gflux->imageWidth);
-    for(i=0;i<gflux->imageHeight;i++) {
-        for(j=0;j<gflux->imageWidth;j++) {
-            c = (((i)%2 ^ (j)%2) ? 100 : 200 );
-            presult(i,j,0) = (GLubyte) c;
-            presult(i,j,1) = (GLubyte) c;
-            presult(i,j,2) = (GLubyte) c;
-            presult(i,j,3) = (GLubyte) 255;
-        }
-    }
-    gflux->image = result;
+  glGenTextures (1, &gflux->texName);
+  glBindTexture (GL_TEXTURE_2D, gflux->texName);
+
+  glTexImage2D (GL_TEXTURE_2D, 0, 3, size, size, 0,
+                GL_RGBA, GL_UNSIGNED_BYTE, data);
+
+  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
+  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
+
+  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
+  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
 }
 
-/* specifies image as texture */
-void initTexture(void)
-{
-    clear_gl_error();
-    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
-    glGenTextures(1, &gflux->texName);
-    glBindTexture(GL_TEXTURE_2D, gflux->texName);
-    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
-    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
-    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
-    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
-    check_gl_error("texture parameter");
-
-    /* Bail out if the texture is too large. */
-    {
-      GLint width;
-      glTexImage2D(GL_PROXY_TEXTURE_2D, 0, GL_RGBA, gflux->imageWidth,
-                   gflux->imageHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, 0);
-      glGetTexLevelParameteriv (GL_PROXY_TEXTURE_2D, 0,
-                                GL_TEXTURE_WIDTH, &width);
-      if (width <= 0)
-        {
-          glGetIntegerv (GL_MAX_TEXTURE_SIZE, &width);
-          fprintf (stderr,
-                   "%s: texture too large (%dx%d -- probable max %dx%d)\n",
-                   progname, gflux->imageWidth, gflux->imageHeight,
-                   width, width);
-          return;
-        }
-    }
-
-    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, gflux->imageWidth,
-            gflux->imageHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, gflux->image);
-    check_gl_error("texture creation");
-}
-
-#undef presult
-#endif
-
 
 void
 grabTexture(void)
 {
-  int real_width  = gflux->modeinfo->xgwa.width;
-  int real_height = gflux->modeinfo->xgwa.height;
-  XImage *ximage = screen_to_ximage (gflux->modeinfo->xgwa.screen,
-                                     gflux->window);
-  Bool bigimage = False;
-  int size = 0;
+  Bool mipmap_p = True;
+  int iw, ih, tw, th;
 
-  if (ximage->width > 1280 ||   /* that's too damned big... */
-      ximage->height > 1280)
-    {
-      Display *dpy = gflux->modeinfo->dpy;
-      Visual *v = gflux->modeinfo->xgwa.visual;
-      int real_size = (ximage->width < ximage->height ?
-                       real_width : real_height);
-      XImage *x2;
-      int x, y, xoff, yoff;
-      size = (ximage->width < ximage->height ?
-              ximage->width : ximage->height);
-      bigimage = True;
+  if (MI_IS_WIREFRAME(gflux->modeinfo))
+    return;
 
-      if (size > 1024) size = 1024;
+  if (! screen_to_texture (gflux->modeinfo->xgwa.screen,
+                           gflux->modeinfo->window, 0, 0, mipmap_p,
+                           NULL, &gflux->img_geom, &iw, &ih, &tw, &th))
+    exit (1);
 
-      x2 = XCreateImage (dpy, v, 32, ZPixmap, 0, 0, size, size, 32, 0);
-      xoff = (real_width  > size ? (random() % (real_width  - size)) : 0);
-      yoff = (real_height > size ? (random() % (real_height - size)) : 0);
-
-# if 0
-      fprintf(stderr, "%s: cropping texture from %dx%d to %dx%d @ %d,%d\n",
-              progname, ximage->width, ximage->height, x2->width, x2->height,
-              xoff, yoff);
-# endif
-      x2->data = ximage->data;  /* we can reuse the same array */
-      for (y = 0; y < x2->height; y++)
-        for (x = 0; x < x2->width; x++)
-          XPutPixel (x2, x, y, XGetPixel (ximage, x+xoff, y+yoff));
-
-      real_width = real_height = real_size;
-      ximage->data = 0;
-      XDestroyImage (ximage);
-      ximage = x2;
-    }
-
-  /* Add a border. */
-  {
-    unsigned long gray = 0xAAAAAAAAL;  /* so shoot me */
-    int width  = (bigimage ? size : real_width);
-    int height = (bigimage ? size : real_height);
-    int i;
-    for (i = 0; i < real_height; i++)
-      {
-        XPutPixel (ximage, 0, i, gray);
-        XPutPixel (ximage, width-1, i, gray);
-      }
-    for (i = 0; i < real_width; i++)
-      {
-        XPutPixel (ximage, i, 0, gray);
-        XPutPixel (ximage, i, height-1, gray);
-      }
-  }
-
-  gflux->imageWidth  = ximage->width;
-  gflux->imageHeight = ximage->height;
-  gflux->image = ximage->data;
-
-  if (bigimage)  /* don't scale really large images */
-    {
-      gflux->tex_xscale = 1;
-      gflux->tex_yscale = 1;
-    }
-  else
-    {
-      gflux->tex_xscale = ((GLfloat) real_width  / (GLfloat) ximage->width);
-      gflux->tex_yscale = ((GLfloat) real_height / (GLfloat) ximage->height);
-    }
-
-  ximage->data = 0;
-  XDestroyImage (ximage);
+  gflux->tex_xscale =  (GLfloat) iw / tw;
+  gflux->tex_yscale = -(GLfloat) ih / th;
+  gflux->img_width  = iw;
+  gflux->img_height = ih;
+   
+  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
+                   (mipmap_p ? GL_LINEAR_MIPMAP_LINEAR : GL_LINEAR));
 }
 
 
@@ -707,6 +500,31 @@ void displayTexture(void)
     double xs = gflux->tex_xscale;
     double ys = gflux->tex_yscale;
 
+    double minx, miny, maxx, maxy;
+    double minu, minv;
+
+#if 0
+    minx = (GLfloat) gflux->img_geom.x / gflux->img_width;
+    miny = (GLfloat) gflux->img_geom.y / gflux->img_height;
+    maxx = ((GLfloat) (gflux->img_geom.x + gflux->img_geom.width) /
+            gflux->img_width);
+    maxy = ((GLfloat) (gflux->img_geom.y + gflux->img_geom.height) /
+            gflux->img_height);
+    minu = minx;
+    minv = miny;
+    minx = (minx * 2) - 1;
+    miny = (miny * 2) - 1;
+    maxx = (maxx * 2) - 1;
+    maxy = (maxy * 2) - 1;
+#else
+    minx = -1;
+    miny = -1;
+    maxx = 1;
+    maxy = 1;
+    minv = 0;
+    minu = 0;
+#endif
+
 	glMatrixMode (GL_TEXTURE);
 	glLoadIdentity ();
 	glTranslatef(-1,-1,0);
@@ -714,6 +532,7 @@ void displayTexture(void)
 	glMatrixMode (GL_MODELVIEW);
 
     glLoadIdentity();
+    userRot();
     glRotatef(anglex,1,0,0);
     glRotatef(angley,0,1,0);
     glRotatef(anglez,0,0,1);
@@ -728,13 +547,11 @@ void displayTexture(void)
 
 	glColor3f(0.5,0.5,0.5);
  
-    for(x=-1,u= 0;x<0.9999;x+=dx,u+=du) {
+    for(x = minx, u = minu; x < maxx - 0.01; x += dx, u += du) {
         glBegin(GL_QUAD_STRIP);
-        for(y=-1,v= 0;y<=1;y+=dy,v+=dv) {
+        for (y = miny, v = minv; y <= maxy + 0.01; y += dy, v += dv) {
             z = getGrid(x,y,time);
-        /*  genColour(z);
-            glColor3fv(gflux->colour);
-        */  glTexCoord2f(u*xs,v*ys);
+            glTexCoord2f(u*xs,v*ys);
             glNormal3f(
                 getGrid(x+dx,y,time)-getGrid(x-dx,y,time),
                 getGrid(x,y+dy,time)-getGrid(x,y-dy,time),
@@ -743,9 +560,7 @@ void displayTexture(void)
             glVertex3f(x,y,z);
 
             z = getGrid(x+dx,y,time);
-        /*  genColour(z);
-            glColor3fv(gflux->colour);
-        */  glTexCoord2f((u+du)*xs,v*ys);
+            glTexCoord2f((u+du)*xs,v*ys);
             glNormal3f(
                 getGrid(x+dx+dx,y,time)-getGrid(x,y,time),
                 getGrid(x+dx,y+dy,time)-getGrid(x+dx,y-dy,time),
@@ -756,10 +571,35 @@ void displayTexture(void)
         glEnd();
     }
 
-    time -= _speed;
-    anglex -= _rotationx;
-    angley -= _rotationy;
-    anglez -= _rotationz;
+    /* Draw a border around the grid.
+     */
+    glColor3f(0.4, 0.4, 0.4);
+    glDisable(GL_TEXTURE_2D);
+    glEnable (GL_LINE_SMOOTH);
+
+    glBegin(GL_LINE_LOOP);
+    y = miny;
+    for (x = minx; x <= maxx; x += dx)
+      glVertex3f (x, y, getGrid (x, y, time));
+    x = maxx;
+    for (y = miny; y <= maxy; y += dy)
+      glVertex3f (x, y, getGrid (x, y, time));
+    y = maxy;
+    for (x = maxx; x >= minx; x -= dx)
+      glVertex3f (x, y, getGrid (x, y, time));
+    x = minx;
+    for (y = maxy; y >= miny; y -= dy)
+      glVertex3f (x, y, getGrid (x, y, time));
+
+    glEnd();
+    glEnable(GL_TEXTURE_2D);
+
+    if (! button_down_p) {
+      time -= _speed;
+      anglex -= _rotationx;
+      angley -= _rotationy;
+      anglez -= _rotationz;
+    }
 }
 void displaySolid(void)
 {
@@ -777,6 +617,7 @@ void displaySolid(void)
     glRotatef(anglex,1,0,0);
     glRotatef(angley,0,1,0);
     glRotatef(anglez,0,0,1);
+    userRot();
     glScalef(1,1,(GLfloat)_waveHeight);
     glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
 
@@ -796,10 +637,12 @@ void displaySolid(void)
         glEnd();
     }
 
-    time -= _speed;
-    anglex -= _rotationx;
-    angley -= _rotationy;
-    anglez -= _rotationz;
+    if (! button_down_p) {
+      time -= _speed;
+      anglex -= _rotationx;
+      angley -= _rotationy;
+      anglez -= _rotationz;
+    }
 
 }
 
@@ -819,6 +662,7 @@ void displayLight(void)
     glRotatef(anglex,1,0,0);
     glRotatef(angley,0,1,0);
     glRotatef(anglez,0,0,1);
+    userRot();
     glScalef(1,1,(GLfloat)_waveHeight);
     glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
 
@@ -848,10 +692,12 @@ void displayLight(void)
         glEnd();
     }
 
-    time -= _speed;
-    anglex -= _rotationx;
-    angley -= _rotationy;
-    anglez -= _rotationz;
+    if (! button_down_p) {
+      time -= _speed;
+      anglex -= _rotationx;
+      angley -= _rotationy;
+      anglez -= _rotationz;
+    }
 }
 
 void displayWire(void)
@@ -872,6 +718,7 @@ void displayWire(void)
     glRotatef(anglex,1,0,0);
     glRotatef(angley,0,1,0);
     glRotatef(anglez,0,0,1);
+    userRot();
     glScalef(1,1,(GLfloat)_waveHeight);
     glClear(GL_COLOR_BUFFER_BIT);
 
@@ -896,10 +743,12 @@ void displayWire(void)
         glEnd();
     }
 
-    time -= _speed;
-    anglex -= _rotationx;
-    angley -= _rotationy;
-    anglez -= _rotationz;
+    if (! button_down_p) {
+      time -= _speed;
+      anglex -= _rotationx;
+      angley -= _rotationy;
+      anglez -= _rotationz;
+    }
 }
 
 /* generates new ripples */
@@ -909,12 +758,14 @@ void calcGrid(void)
     double tmp;
     static int newWave;
 
+    if (button_down_p) return;
+
     tmp = 1.0/((double)_waveChange);
     if(!(counter%_waveChange)) {
         newWave = ((int)(counter*tmp))%_waves;
-        gflux->dispx[newWave] = 1.0 - ((double)random())/RAND_MAX;
-        gflux->dispy[newWave] = 1.0 - ((double)random())/RAND_MAX;
-        gflux->freq[newWave] = _waveFreq * ((float)random())/RAND_MAX;
+        gflux->dispx[newWave] = -frand(1.0);
+        gflux->dispy[newWave] = -frand(1.0);
+        gflux->freq[newWave] = _waveFreq * frand(1.0);
         gflux->wa[newWave] = 0.0;
     }
     counter++;
