@@ -1,9 +1,8 @@
 /* -*- Mode: C; tab-width: 4 -*- */
 /* superquadrics --- 3D mathematical shapes */
 
-#if !defined( lint ) && !defined( SABER )
+#if 0
 static const char sccsid[] = "@(#)superquadrics.c	4.07 97/11/24 xlockmore";
-
 #endif
 
 /*-
@@ -71,21 +70,15 @@ static const char sccsid[] = "@(#)superquadrics.c	4.07 97/11/24 xlockmore";
  * Ed Mackey
  */
 
-/*-
- * due to a Bug/feature in VMS X11/Intrinsic.h has to be placed before xlock.
- * otherwise caddr_t is not defined correctly
- */
-
-#include <X11/Intrinsic.h>
-
 #ifdef STANDALONE
 # define PROGCLASS					"Superquadrics"
 # define HACK_INIT					init_superquadrics
 # define HACK_DRAW					draw_superquadrics
 # define superquadrics_opts			xlockmore_opts
-# define DEFAULTS	"*delay:		100     \n"			\
+# define DEFAULTS	"*delay:		40000   \n"			\
 					"*count:		25      \n"			\
 					"*cycles:		40      \n"			\
+					"*showFPS:      False   \n"			\
 					"*wireframe:	False	\n"
 # include "xlockmore.h"				/* from the xscreensaver distribution */
 #else  /* !STANDALONE */
@@ -107,11 +100,11 @@ static float spinspeed;
 
 static XrmOptionDescRec opts[] =
 {
-  {"-spinspeed", ".superquadrics.spinspeed", XrmoptionSepArg, (caddr_t) NULL}
+  {"-spinspeed", ".superquadrics.spinspeed", XrmoptionSepArg, 0}
 };
 static argtype vars[] =
 {
-  {(caddr_t *) & spinspeed, "spinspeed", "Spinspeed", DEF_SPINSPEED, t_Float}
+  {&spinspeed, "spinspeed", "Spinspeed", DEF_SPINSPEED, t_Float}
 };
 static OptionStruct desc[] =
 {
@@ -752,6 +745,7 @@ draw_superquadrics(ModeInfo * mi)
 
 	NextSuperquadricDisplay(sp);
 
+    if (mi->fps_p) do_fps (mi);
 	glFinish();
 	glXSwapBuffers(display, window);
 }
