@@ -1,9 +1,8 @@
 /* -*- Mode: C; tab-width: 4 -*- */
 /* lisa --- animated full-loop lisajous figures */
 
-#if !defined( lint ) && !defined( SABER )
+#if 0
 static const char sccsid[] = "@(#)lisa.c	5.00 2000/11/01 xlockmore";
-
 #endif
 
 /*-
@@ -63,18 +62,18 @@ static Bool additive;
 
 static XrmOptionDescRec opts[] =
 {
-	{(char *) "-additive", (char *) ".lisa.additive", XrmoptionNoArg, (caddr_t) "True"},
-	{(char *) "+additive", (char *) ".lisa.additive", XrmoptionNoArg, (caddr_t) "False"}
+	{"-additive", ".lisa.additive", XrmoptionNoArg, "True"},
+	{"+additive", ".lisa.additive", XrmoptionNoArg, "False"}
 };
 
 static argtype vars[] =
 {
-	{(caddr_t *) & additive, (char *) "additive", (char *) "Additive", (char *) DEF_ADDITIVE, t_Bool}
+	{&additive, "additive", "Additive", DEF_ADDITIVE, t_Bool}
 };
 
 static OptionStruct desc[] =
 {
-	{(char *) "-/+additive", (char *) "turn on/off additive functions mode"}
+	{"-/+additive", "turn on/off additive functions mode"}
 };
 
 ModeSpecOpt lisa_opts =
