@@ -1,9 +1,8 @@
 /* -*- Mode: C; tab-width: 4 -*- */
 /* loop --- Chris Langton's self-producing loops */
 
-#if !defined( lint ) && !defined( SABER )
+#if 0
 static const char sccsid[] = "@(#)loop.c	5.01 2000/03/15 xlockmore";
-
 #endif
 
 /*-
@@ -91,7 +90,7 @@ static const char sccsid[] = "@(#)loop.c	5.01 2000/03/15 xlockmore";
  "*cycles: 1600 \n" \
  "*size: -12 \n" \
  "*ncolors: 15 \n" \
- "*neighbors: 0 \n"
+
 #define UNIFORM_COLORS
 #include "xlockmore.h"		/* in xscreensaver distribution */
 #else /* STANDALONE */
@@ -110,17 +109,17 @@ static int  neighbors;
 
 static XrmOptionDescRec opts[] =
 {
-	{(char *) "-neighbors", (char *) ".loop.neighbors", XrmoptionSepArg, (caddr_t) NULL}
+	{"-neighbors", ".loop.neighbors", XrmoptionSepArg, 0}
 };
 
 static argtype vars[] =
 {
-	{(caddr_t *) & neighbors, (char *) "neighbors", (char *) "Neighbors", (char *) DEF_NEIGHBORS, t_Int}
+	{&neighbors, "neighbors", "Neighbors", DEF_NEIGHBORS, t_Int}
 };
 
 static OptionStruct desc[] =
 {
-	{(char *) "-neighbors num", (char *) "squares 4 or hexagons 6"}
+	{"-neighbors num", "squares 4 or hexagons 6"}
 };
 
 ModeSpecOpt loop_opts =
@@ -1426,6 +1425,9 @@ release_loop(ModeInfo * mi)
 	}
 }
 
+static void *stop_warning_about_triangleUnit_already;
+
+
 void
 init_loop(ModeInfo * mi)
 {
@@ -1435,6 +1437,8 @@ init_loop(ModeInfo * mi)
 	loopstruct *lp;
 	XGCValues   gcv;
 
+    stop_warning_about_triangleUnit_already = (void *) &triangleUnit;
+
 	if (loops == NULL) {
 		if ((loops = (loopstruct *) calloc(MI_NUM_SCREENS(mi),
 					       sizeof (loopstruct))) == NULL)
