@@ -1,5 +1,4 @@
-/* xscreensaver, Copyright (c) 1992, 1993, 1994, 1996, 1997, 1998, 2001 
- * Jamie Zawinski <jwz@jwz.org>
+/* xscreensaver, Copyright (c) 1992-2005 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -36,7 +35,7 @@ init_slide (Display *dpy, Window window)
   Visual *visual;
 
   XGetWindowAttributes (dpy, window, &xgwa);
-  grab_screen_image (xgwa.screen, window);
+  load_random_image (xgwa.screen, window, window, NULL, NULL);
   cmap = xgwa.colormap;
   visual = xgwa.visual;
   max_width = xgwa.width;
@@ -98,7 +97,7 @@ init_slide (Display *dpy, Window window)
 	for(i = 0; i < max; i++)
 	  {
 	    long rd, gd, bd;
-	    unsigned long d;
+	    unsigned long dd;
 	    if (!fg_ok)
 	      {
 		rd = (all[i].red   >> 8) - (fgc.red   >> 8);
@@ -107,12 +106,12 @@ init_slide (Display *dpy, Window window)
 		if (rd < 0) rd = -rd;
 		if (gd < 0) gd = -gd;
 		if (bd < 0) bd = -bd;
-		d = (rd << 1) + (gd << 2) + bd;
-		if (d < fgd)
+		dd = (rd << 1) + (gd << 2) + bd;
+		if (dd < fgd)
 		  {
-		    fgd = d;
+		    fgd = dd;
 		    fg = all[i].pixel;
-		    if (d == 0)
+		    if (dd == 0)
 		      fg_ok = True;
 		  }
 	      }
@@ -125,12 +124,12 @@ init_slide (Display *dpy, Window window)
 		if (rd < 0) rd = -rd;
 		if (gd < 0) gd = -gd;
 		if (bd < 0) bd = -bd;
-		d = (rd << 1) + (gd << 2) + bd;
-		if (d < bgd)
+		dd = (rd << 1) + (gd << 2) + bd;
+		if (dd < bgd)
 		  {
-		    bgd = d;
+		    bgd = dd;
 		    bg = all[i].pixel;
-		    if (d == 0)
+		    if (dd == 0)
 		      bg_ok = True;
 		  }
 	      }
