@@ -1,4 +1,5 @@
-/* xscreensaver, Copyright (c) 1992, 1997, 2001 Jamie Zawinski <jwz@jwz.org>
+/* xscreensaver, Copyright (c) 1992, 1997, 2001, 2003
+ *  Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -22,7 +23,8 @@ extern unsigned int get_seconds_resource (char*,char*);
 extern int parse_time (const char *string, Bool seconds_default_p,
                        Bool silent_p);
 extern Pixmap
-xscreensaver_logo (Display *dpy, Window window, Colormap cmap,
+xscreensaver_logo (Screen *screen, Visual *visual,
+                   Drawable drawable, Colormap cmap,
                    unsigned long background_color,
                    unsigned long **pixels_ret, int *npixels_ret,
                    Pixmap *mask_ret,
